# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the License.
#
# Authors:
# -----------------------------------------------------------------------------
#   Rueen Fiez <rueenfiez@gmail.com>
#   Vincent Lam <vlam6@learn.senecac.on.ca>
#   Armen Zambrano Gasparnian <armenzg@gmail.com>
# -----------------------------------------------------------------------------
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****

# VERSION:
# ==============================
# version 0.8
# ==============================

from optparse import OptionParser
import shutil
import re
import os
import os.path
import Parser
from Parser import DTDParser
from Parser import PropertiesParser

import sys

class Tool:
     
     #constructor
     def __init__(self):
          self.data = []          

     # DESCRIPTION
     # callback() is passed a string to be translated. It is responsible for changing all parser.contents
     # values. We insert the translation rules in here via regular expressions. The searchKey param is
     # used to identify when callback needs to localize a Search Default in the tree. eg; Google = 1. 
     def callback(self, instring, searchKey=0):
          # if we use subn(), which returns a 2-tuple containing
          # the new string value and the number of replacements
          # that were performed
          # DOING so will allow us to know if the file has been modify
          #reg1 = re.compile(r'(.*".*)([Dd])ialogue')
          #reg2 = re.compile(r'(.*".*)([Cc])olor')
          #reg3 = re.compile(r'(.*".*)([Gg])o forward')
          #reg4 = re.compile(r'(.*".*)([Mm])inimize')
          #reg5 = re.compile(r'(.*".*)([Cc])enter')
          #reg6 = re.compile(r'(.*".*)([Oo])rganize')
          #reg7 = re.compile(r'(.*".*)([Cc])ustomize')
          #instring = reg1.sub(r'\1\2ialog', instring)
          #instring = reg2.sub(r'\1\2olour', instring)
          #instring = reg3.sub(r'\1\2o forwards', instring)
          #instring = reg4.sub(r'\1\2inimise', instring)
          #instring = reg5.sub(r'\1\2entre', instring)
          #instring = reg6.sub(r'\1\2rganise', instring)
          #instring = reg7.sub(r'\1\2ustomise', instring)
          
          #instring = re.sub(r'(.*)[Cc]olor', r'\1\2olour', instring)

          IN_FILE = open('locale_template.properties', "r")
          while True:
               kvLine = IN_FILE.readline()
               if not kvLine:
                    break
               key, value = kvLine.strip().split("=")

               #### SEARCH ENGINE LOCALIZATION LOGIC ####

               # When we find a "google.xml" in the source tree, searchKey equals '1'
               if (searchKey == 1 and key == "googleURL"):
                    instring = re.sub(r'google\.com', value, instring).strip()

               # When we find a "wikipedia.xml" in the source tree, searchKey equals '2'
               if (searchKey == 2 and key == "wikipediaURL"):
                    instring = re.sub(r'en\.wikipedia\.org', value, instring).strip()

               # When we find a "yahoo.xml" in the source tree, searchKey equals '3'
               if (searchKey == 3 and key == "yahooURL"):
                    instring = re.sub(r'yahoo\.com', value, instring).strip()

               # When we find a "amazondotcom.xml" in the source tree, searchKey equals '4'
               if (searchKey == 4 and key == "amazonURL"):
                    instring = re.sub(r'amazon\.com', value, instring).strip()

               # When we find a "eBay.xml" in the source tree, searchKey equals '5'
               if (searchKey == 5 and key == "ebayURL"):
                    instring = re.sub(r'ebay\.com', value, instring).strip()

               ##########################################                    
                    
               # Used for DEBUGGING
               #print "KEY= " + key + "\tVALUE= " + value
               key = key.replace(key[0], r"(.*)([" + key[0].upper() + key[0].lower() + "])")
               #key = key.replace(key[len(key) - 1], key[len(key) - 1] + "'")
               # Used for DEBUGGING
               #print key
               value = value.replace(value[0], r"\1\2")
               #value = value.replace(value[len(value) - 1], key[len(key) - 1] + "'")
               # Used for DEBUGGING
               #print value
               instring = re.sub(key, value, instring)
          IN_FILE.close()
          
          return instring

     # DESCRIPTION
     # translate() uses the DTDParser and PropertiesParser from the Parser.py class to
     # parse each .dtd and .properties file in the given directory. Its contents
     # are then passed into our callback() function string by string and changed in their.
     def translate(self, node):
          dtdParser = DTDParser()
          propParser = PropertiesParser()
          if(node.endswith('.dtd')):
              f = open(node,'r+')
              dtdParser.read(node)
              f.write((t.callback(dtdParser.contents)).encode("utf-8"))
              # Use for DEBUGGING
              if options.debug == True:
                   print "DTD_PARSER_CONTENTS: \n" + dtdParser.contents
              f.close()
          elif(node.endswith('.properties')):
              f = open(node,'r+')
              propParser.read(node)
              f.write((t.callback(propParser.contents)).encode("utf-8"))
              # Use for DEBUGGING
              if options.debug == True:
                   print "PROP_PARSER_CONTENTS: \n" + propParser.contents
              f.close()
          elif(node.endswith('.dic')):
              f = open(node,'r+')
              propParser.read(node)
              f.write((t.callback(propParser.contents)).encode("utf-8"))
              # Use for DEBUGGING
              if options.debug == True:
                   print "DICTIONARY_PARSER_CONTENTS: \n" + propParser.contents
              f.close()
          elif(node.endswith('google.xml')):
               # Used for DEBUGGING
               #print "***********FOUND GOOGLE XML************** \n"
               f = open(node,'r+')
               propParser.read(node)
               f.write((t.callback(propParser.contents, 1)).encode("utf-8"))
               # Use for DEBUGGING
               if options.debug == True:
                    print "GOOGLEXML_PARSER_CONTENTS: \n" + propParser.contents
               f.close()
          elif(node.endswith('wikipedia.xml')):
               # Used for DEBUGGING
               #print "***********FOUND WIKIPEDIA XML************** \n"
               f = open(node,'r+')
               propParser.read(node)
               f.write((t.callback(propParser.contents, 2)).encode("utf-8"))
               # Use for DEBUGGING
               if options.debug == True:
                    print "WIKIPEDIA_PARSER_CONTENTS: \n" + propParser.contents
               f.close()
          elif(node.endswith('yahoo.xml')):
               # Used for DEBUGGING
               #print "***********FOUND YAHOO XML************** \n"
               f = open(node,'r+')
               propParser.read(node)
               f.write((t.callback(propParser.contents, 3)).encode("utf-8"))
               # Use for DEBUGGING
               if options.debug == True:
                    print "YAHOO_PARSER_CONTENTS: \n" + propParser.contents
               f.close()
          elif(node.endswith('amazondotcom.xml')):
               # Used for DEBUGGING
               #print "***********FOUND AMAZON XML************** \n"
               f = open(node,'r+')
               propParser.read(node)
               f.write((t.callback(propParser.contents, 4)).encode("utf-8"))
               # Use for DEBUGGING
               if options.debug == True:
                    print "AMAZON_PARSER_CONTENTS: \n" + propParser.contents
               f.close()
          elif(node.endswith('eBay.xml')):
               # Used for DEBUGGING
               #print "***********FOUND EBAY XML************** \n"
               f = open(node,'r+')
               propParser.read(node)
               f.write((t.callback(propParser.contents, 5)).encode("utf-8"))
               # Use for DEBUGGING
               if options.debug == True:
                    print "EBAY_PARSER_CONTENTS: \n" + propParser.contents
               f.close()
               

     # DESCRIPTION
     # process() is the function responsible for actually "walking" a
     # given directory. It contains the logic used to see if each item in the folder
     # is either a "file" or a "directory".
     def process(self, forkedLocale):
          directories = [forkedLocale]
          while len(directories) > 0: #while there are directories to walk through
              directory = directories.pop() #get one of the directories reachable from CWD
              # Used for DEBUGGING
              if options.verbose == True:
                   print directory
              for name in os.listdir(directory): # we are going to iterate through every element of directory
                  fullpath = os.path.join(directory,name)
                  if os.path.isfile(fullpath):
				      #If is a file, try to make changes ot it
                      # Used for DEBUGGING
                           #print "FULLPATH: " + fullpath
                      t.translate(fullpath)
                  elif os.path.isdir(fullpath): #if it is a dir, let's append it to iterate later on
                      directories.append(fullpath)

     # Will create template.
     def createTemplate(self):
          # Create a List of initial Keys to be added to the localization file. More Keys can be appended to the file manually or hardcoded right here.
          initKeys = ['color', 'center', 'googleURL']
          # Turn the collection into a Set to avoid duplicates
          st_initKeys = set(initKeys)

          # Used for DEBUGGING
          #print st_initKeys
          
          # The name of the template file --> 'locale_template.properties'
          filename = "locale_template.properties"

          # Check to see if the 'locale_template.properties' file already exists
          if os.path.exists(filename):
               print "The localization template file already exists in your current working directory.\n"
          else:
               # Localizers can append more Keys to the Localization file manually if needed. But system provides some initial Keys to work with.
               # Create the Localization file and start writing
               FILE = open(filename,'w')

               for k in st_initKeys:
                    FILE.write(k + "=" + "\n") # Write the Key, then an equal '=' then add a newline
               
               # TODO: Create a class that has a method which processes a bunch of FILE.writeline(...) CMDs when run.
               FILE.close()
               #print "A new localization template has been generated in the same location as this script.\n"

     def importTemplate(self):
          print "Hi"

if __name__ == '__main__':

     # Parse command line arguments. Available options below
     parser = OptionParser()

     parser.add_option("-d", "--debug", action="store_true", dest="debug", help="Shows debug details of forking process\n")
     parser.add_option("-v", "--verbose", action="store_true", dest="verbose", help="Shows the absolute path of the files being parsed\n")
     parser.add_option("-t", "--template", action="store_true", dest="template", help="Create a localization template in your cwd\n")
     #parser.add_option("-i", "--import", action="store", dest="locale_Template", help="Use this option to import a localization template. To generate one, use the \"-t\" flag\n")
     parser.add_option("-s", "--source", action="store", type="string", dest="source", help="The original source that will be forked\n")
     parser.add_option("-n", "--new", action="store", type="string", dest="newSource", help="The name given to the new forked tree\n")
     (options, args) = parser.parse_args()

     # DEBUG
     #print "Template value: ", options.template

     # Create an instance of the tool
     t = Tool()

     # If user wants to generate a template, run the createTemplate() method. If not, it means it's a simple localization command
     if options.template == True:
          t.createTemplate()
     else:
          if (re.search('^\w',options.source) and re.search('^\w', options.newSource)):
               # Copy original tree. The '^' means starts with and the '\w' is short for [a-zA-Z0-9]
               shutil.copytree(options.source, options.newSource)

               # Process changes to new tree
               t.process(options.newSource)  # Validation needed here to see if new forked tree directory exists. Code below for this. [For 0.5 Release]

               # When complete, system returns msg
               #print "\nProcessing completed. Check current working directory for " + options.newSource + " source tree.\n"

               # DEBUG
               #print len(options.source)
          else:
               # If in here, it means either the source or newSource arguments were invalid
               print "ERROR: source or new-source name must begin with an alphanumeric character."
     
     
     # ###################################################################################
     # Code below was our original "main" code. Good for quick & dirty testing
     # Version 0.1 - [Deprecated]
     #
     # Original #
     #t = Tool()
     #t.process('en-GB')
     #print raw_input("press Return>")
     #
     # ###################################################################################

     #####################################################################################
     # Version 0.3.2 Main - [Working]
     #
     #t = Tool()
     # Parse command line arguments
     # TODO We could use OptionParser() rather than this way
     #forkedLocale = raw_input("Enter original l10n tree (for now just type 'en')\nPath: ")
     # Some input validation
     #if len(forkedLocale) >= 2: #TODO a regex will make more sense
     #    newLocaleNameFlag = False
     #    while(newLocaleNameFlag == False):
     #         newLocaleName = raw_input("Enter a new locale-regional name (eg; en-CA)\nNew Locale Name: ")
     #         if os.path.exists(newLocaleName) == False:
     #              newLocaleNameFlag = True
     #              shutil.copytree(forkedLocale, newLocaleName)
     #              t.process(newLocaleName)
     #         else:
     #              print "The directory " + newLocaleName + " already exists. Please choose another one.\n"
     #
     #    print raw_input("\nTranslation(s) for " + newLocaleName + " complete. Press ENTER to exit...>")
     #else:
     #    print "\nNeed to input a valid localization path!"
     #
     #####################################################################################

     
     


